/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

var EXPORTED_SYMBOLS = ["onOverlay", "m_extActive"];
Components.utils.import("resource://sublink-gremio-modules/util.js");


function onOverlay(win) {
  win.addEventListener("load", onLoad, false);
}


function onLoad(evt) {
  var win = evt.currentTarget;
  win.removeEventListener("load", onLoad, false);
  win.addEventListener("unload", onUnload, false);


  if (m_extActive) {
    enableExtWin(win.document);
  }

  if (m_doFirstRun) {
    m_doFirstRun = false;
    Components.utils.import("resource://sublink-gremio-modules/first-run.js");
  }
}


function onUnload(evt) {
  var win = evt.currentTarget;
  win.removeEventListener("unload", onUnload, false);
  if (m_extActive) {
    disableExtWin(win.document);
  }
}


function enableExtWin(doc) {
  var cmdButton = doc.getElementById("cmd_sublink-gremio-copy-url");
  //cmdButton.addEventListener("command", onButtonCommand, false);
  cmdButton.setAttribute("label", util.getText("button.create.label", "grem.io"));
  cmdButton.hidden = false;

  doc.addEventListener("popupshowing", onPopupShowing, false);
  doc.addEventListener("popupshown", onPopupShown, false);
}


function disableExtWin(doc) {
  var cmdButton = doc.getElementById("cmd_sublink-gremio-copy-url");
  //cmdButton.removeEventListener("command", onButtonCommand, false);
  cmdButton.hidden = true;

  doc.removeEventListener("popupshowing", onPopupShowing, false);
  doc.removeEventListener("popupshown", onPopupShown, false);
}


function enableExt(enable) {
  m_extActive = enable;
  var winEnum = util.browserWindowsEnum();
  while (winEnum.hasMoreElements()) {
    var doc = winEnum.getNext().document;
    if (enable) {
      enableExtWin(doc);
    } else {
      disableExtWin(doc);
    }
  }
}


function isContextMenu(popup) {
  switch (popup.parentNode.localName) {
    case "menu":
    case "toolbarbutton":
      return false;
    //case "popupset":
  }
  return true;
}


function onPopupShowing(evt) {
  var popup = evt.originalTarget;

  switch (popup.localName) {
    case "tooltip":
      if (popup.ownerDocument.tooltipNode.id === "sublink-gremio-button") {
        Components.utils.import("resource://sublink-gremio-modules/button.js");
        buttonTooltip(popup);
      }
      break;

    case "menupopup":
    case "popup":
      util.log("onPopupShowing triggerNode=" + popup.triggerNode);

      if (isContextMenu(popup)) {
        Components.utils.import("resource://sublink-gremio-modules/menupopup.js");
        cmdPopupshowing(popup, evt.target);
      }
      break;
  }
}


// workaround ==> XUL ==> on TwitterFox, document.popupNode in "popupshowing" is not reliable
function onPopupShown(evt) {
  var popup = evt.originalTarget;

  util.log("onPopupShown triggerNode=" + popup.triggerNode);

  switch (popup.localName) {
    case "menupopup":
    case "popup":
      if (isContextMenu(popup)) {
        Components.utils.import("resource://sublink-gremio-modules/menupopup.js");
        cmdPopupshown(popup);
      }
      break;
  }
}

/*
function onButtonCommand(evt) {
  Components.utils.import("resource://sublink-gremio-modules/button.js");
  buttonCmd(evt);
}
*/

var AddonsManager = {
  register: function() {
    Cc["@mozilla.org/observer-service;1"]
      .getService(Ci.nsIObserverService)
      .addObserver(this, "em-action-requested", false);
  },
  /*
  unregister: function() {
    Cc["@mozilla.org/observer-service;1"]
      .getService(Ci.nsIObserverService)
      .removeObserver(this, "em-action-requested");
  },
  */

  observe: function(aSubject, aTopic, aData) {
    aSubject.QueryInterface(Ci.nsIUpdateItem);
    if (aSubject.id !== "grem.io@hultmann") {
      return;
    }

    switch (aData) {
      case "item-enabled":
      case "item-cancel-action":
        enableExt(true);
        break;

      case "item-disabled":
      case "item-uninstalled":
        enableExt(false);
        Components.utils.import("resource://sublink-gremio-modules/redir-storage.js");
        redirStorage.removeOrganizerItem();
        break;

      //case "item-installed":
      //case "item-upgraded":
    }

  }

};


// startup (first overlay)

var m_extActive;
var m_doFirstRun;

(function() {
  m_extActive = true;
  AddonsManager.register();
  var prefs = Cc["@mozilla.org/preferences-service;1"]
                .getService(Ci.nsIPrefService)
                .getBranch("extensions.grem.io@hultmann.");
  if (prefs.getBoolPref("firstRun")){
    prefs.setBoolPref("firstRun", false);
    m_doFirstRun = true;
  } else {
    m_doFirstRun = false;
  }
})();
